#!/usr/bin/env python

"""
    STP Video Encoder Base
    ======================
    Base class for encoding video. Inherit this when writing new encoders.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import os
import shutil
import tempfile
import threading

from ..metadata import Metadata

class Encoder(threading.Thread):
    """
        Encoder Base Class
        ==================
        Base that all encoder classes should inherit. 
    """
    ext = ""
    def __init__(self, infile, prefix, scale, vbitrate, abitrate):
        threading.Thread.__init__(self)
        self.setDaemon(True)
        self.infile = infile
        self.prefix = prefix
        self.outfile = prefix + self.ext
        self.scale = scale
        self.vbitrate = vbitrate
        self.abitrate = abitrate
    
    def run(self):
        """
            Start this encoder. Use the start() method to run in a separate
            thread.
        """
        tmpdir = tempfile.mkdtemp()
        olddir = os.getcwd()
        os.chdir(tmpdir)
        
        self.metadata = Metadata(self.infile)
        self.width, self.height = self.metadata.get_scaled_dimensions(self.scale)
        
        self.encode()
        
        os.chdir(olddir)
        shutil.rmtree(tmpdir)
    
    def encode(self):
        """
            You should do actual encoding in this method. Override it in your
            own classes.
        """
        raise NotImplementedError

