#!/usr/bin/env python

"""
    STP Video Encoder Thumbnailer
    =============================
    Generate a thumbnail from a video!
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging

from .. import settings

from ..utils import exec_with_progress

from ..encoders.base import Encoder

log = logging.getLogger("libstpenc.encoders.thumb")

BIN_FFMPEG = settings.BIN_FFMPEG

class ThumbEnc(Encoder):
    """
        Thumbnail encoder.
    """
    ext = "-thumb.jpg"
    def encode(self):
        log.info("Creating thumbnail conversion...")
        
        position = min(20, self.metadata.duration / 2)
        
        cmd = "%s -y -i \"%s\" -s %dx%d -ss %d -f image2 -vframes 1 \"%s\"" % \
                 (BIN_FFMPEG, self.infile, self.width, self.height, 
                 position, self.outfile)
        
        exec_with_progress(cmd, self.metadata)
        
        log.debug("Thumbnail generated.")

settings.FORMATS["thumb"] = ThumbEnc

