#!/usr/bin/env python

"""
    STP Video Encoder FTP/FTPS Uploader
    ===================================
    Uploader using FTP and FTPS (FTP via SSL). You should ALWAYS use secure
    connections, FTP is provided as a fallback for services that only support
    insecure connections.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import os
import sys

from ftplib import FTP

import libstpenc.settings

import ftptls

log = logging.getLogger("libstpenc.uploaders.ftp")

def transfer_ftp(host, user, password, src, dst, tls=True):
    """
        Send a file src on the local machine to a file dst on host using the
        given user/pass via ftp. If the tls option is False, then unencrypted
        ftp is used rather than using TLS/SSL to protect the username/password.
        You should NEVER disable tls.
    """
    if user and not tls:
        log.warning("*" * 70)
        log.warning("FTP is insecure and should never be used when a " \
                    "username and password are required!")
        log.warning("*" * 70)
    
    # Connect to server
    try:
        if tls:
            ftp = ftptls.FTP_TLS(host)
        else:
            ftp = FTP(host)
    except socket.error:
        log.error("Unable to connect to ftp server %s!" % host)
        return
    
    # Authenticate and login
    if tls:
        try:
            ftp.auth_tls()
        except:
            log.error("Unable to initiate TLS session with %s!" % host)
            return
    
    try:
        if user:
            ftp.login(user, password)
        else:
            ftp.login()
    except socket.error:
        log.error("Unable to authenticate with ftp server!")
        ftp.close()
        return
    
    # Switch to a protected (encrypted) mode if tls is active
    if tls:
        try:
            ftp.prot_p()
        except:
            log.error("Unable to switch to protected mode for transfer to %s!" % \
                      host)
    
    # Send the file
    log.info("Uploading file via ftp... (no progress will be shown)")
    ftp.storbinary("STOR " + dst, open(src))
    
    # Close the connection
    ftp.quit()

